package collection;

import java.util.*;

public class LinkedList<T> implements Iterable<T> {

    private Node root = new Node(null, null);
    private long modId = 0;

    private void touch() {
	this.modId++;
    }

    public void add(T elem) {
	this.touch();
	this.root.next = new Node(elem, this.root.next);
    }

    public void clear() {
	if (this.root.next != null) {
	    this.touch();
	    this.root.next = null;
	}
    }

    public Iterator<T> iterator() {
	return new ListIterator();
    }

    public void append(Iterable<? extends T> other) {
	Node last = this.root;
	while (last.next != null) {
	    last = last.next;
	}
	for (T elem : other) {
	    this.touch();
	    last.next = new Node(elem, null);
	    last = last.next;
	}
    }

    public String toString() {
	StringBuilder sb = new StringBuilder();
	sb.append('[');
	for (T elem : this) {
	    sb.append(elem);
	    sb.append(' ');
	}
	sb.append(']');
	return sb.toString();
    }


    class ListIterator implements Iterator<T> {
	private Node curr = LinkedList.this.root.next;
	private long expectedModId = LinkedList.this.modId;

	public boolean hasNext() {
	    if (LinkedList.this.modId != this.expectedModId) {
		throw new ConcurrentModificationException();
	    }
	    return this.curr != null;
	}

	public T next() {
	    if (this.hasNext()) {
		T t = this.curr.elem;
		this.curr = this.curr.next;
		return t;
	    } else {
		throw new NoSuchElementException();
	    }
	}

	public void remove() {
	    throw new UnsupportedOperationException();
	}
    }

    class Node {
	T elem;
	Node next;

	private Node(T elem, Node next) {
	    this.elem = elem;
	    this.next = next;
	}
    }

}
