import java.io.*;
import java.net.*;

public class ResourceLoader {

    public static String loadResource(String url) throws ResourceException {
        BufferedReader r = null;
        try {
            if (url.startsWith("file://")) {
                r = new BufferedReader(new FileReader(url.substring(7)));
            } else if (url.startsWith("http://")) {
                URL u = new URL(url);
                URLConnection c = u.openConnection();
                r = new BufferedReader(
                        new InputStreamReader(c.getInputStream()));
            } else if (url.startsWith("sql://")) {
                throw new ResourceException("currently no database connection implemented");
            } else {
                throw new ResourceException("unknown URL type, please start with http:// or file:// or sql://");
            }
        } catch (FileNotFoundException e) {
            throw new ResourceException(e);
        } catch (MalformedURLException e) {
            throw new ResourceException(e);
        } catch (IOException e) {
            throw new ResourceException(e);
        }        
        String s;
        StringBuilder sb = new StringBuilder();
        try {
            String line = r.readLine();
            while (line != null) {
                sb.append(line); sb.append("\n");
                line = r.readLine();
            }
            s = sb.toString();        
        } catch (IOException e) {
            throw new ResourceException(e);
        } finally {
            try {
                r.close();
            } catch (IOException e) {
                throw new ResourceException(e);
            }            
        }
        return s;
    }

    public static void main(String[] args) {
        try {
            System.out.println(loadResource(args[0]));
        } catch (ResourceException e) {
            e.printStackTrace();
        }
    }

}

class ResourceException extends Exception {
    public ResourceException(String message) {
        super(message);
    }
    
    public ResourceException(Throwable t) {
        super(t);
    }
}

