// a stack implementation with bounded size
public class ArrayStack implements Stack, Container {
  
    private final String[] data;
    private int position = 0;
    
    private final static int DEFAULT_SIZE = 100;

    public ArrayStack() {
        this(DEFAULT_SIZE);
    }
    
    public ArrayStack(int size) {
        this.data = new String[size];
    }
    
    public void push(String s) {
        if (this.position < this.data.length) {
            this.data[this.position] = s;
            this.position++;
        } else {
            System.err.println("could not insert");
        }
    }
    
    public int size() {
        return this.position;
    }
    
    public String pop() {
        if (this.position > 0) {            
            this.position--;
            String s = this.data[this.position];
            this.data[this.position] = null;
            return s;
        } else {
            return null; // or throw error
        }
    }
        
    public void addElement(String s) {
        this.push(s);
    }
    
    public Iterator iterator() {
        return new ArrayIterator(this.data, this.position);
    }
   
}

class ArrayIterator implements Iterator {
    private int position = 0;
    private final int size;
    private final String[] data;
    
    ArrayIterator(String[] data, int size) {
        this.size = size;
        this.data = data;
    }
    
    public boolean hasNext() {
        return this.position < this.size;
    }
    
    public String next() {
        if (this.hasNext()) {
            String s = this.data[this.position];
            this.position++;
            return s;
        } else {
            return null; // or throw error message
        }
    }
}
