public class LinkedList implements Container {
    
    private final Node dummyHead;
    private final Node dummyTail; 
    
    public LinkedList() {
        Node head = new Node(null,null,null);
        Node tail = new Node(null,null,null);
        this.dummyHead = head;
        this.dummyTail = tail;
        head.next = tail;
        tail.prev = head;
    }
        
    public int size() {
        Node pos = this.dummyHead.next;
        int count = 0;
        while (pos != this.dummyTail) {
            pos = pos.next;
            count++;
        }
        return count;
    }
    
    public RemovableIterator iterator() {
        return new LinkedListIterator(this.dummyHead.next, this.dummyTail);
    }
    
    public void addElement(String s) {
        Node n = new Node(s,this.dummyHead,this.dummyHead.next);
        this.dummyHead.next = n;
        n.next.prev = n;        
    }
    
}

class LinkedListIterator implements RemovableIterator {
    
    private Node curr;
    private Node returned = null;
    private final Node last;
    
    LinkedListIterator(Node start, Node last) {
        this.curr = start;
        this.last = last;
    }
    
    public boolean hasNext() {
        return this.curr != last;
    }
    
    public String next() {
        if (this.hasNext()) {
            this.returned = this.curr;
            this.curr = this.curr.next;
            return this.returned.data;
        } else {
            return null;
        }
    }
    
    public void remove() {
        if (this.returned != null) {
            Node prev = this.returned.prev;
            Node next = this.returned.next;
            prev.next = next;
            next.prev = prev;
            this.returned = null;
        } else {
            System.err.println("invoke next before remove");
        }
    }
}

class Node {
    
    final String data;
    Node prev;
    Node next;
    
    Node(String data, Node prev, Node next) {
        this.data = data;
        this.prev = prev;
        this.next = next;
    }
}